@@ -1,19 +1,22 @@
-# This program source code file is part of KiCad, a free EDA CAD application.
+# Copyright The KiCad Developers
 #
-# Copyright (C) 2024 KiCad Developers
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the “Software”), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
 #
-# This program is free software: you can redistribute it and/or modify it
-# under the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or (at your
-# option) any later version.
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
 #
-# This program is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License along
-# with this program.  If not, see <http://www.gnu.org/licenses/>.
+# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
 
 from typing import Dict, Sequence, Optional, Union
 from google.protobuf.message import Message
@@ -22,7 +25,7 @@ from google.protobuf.any_pb2 import Any
 from kipy.proto.common.types import KIID
 from kipy.proto.common.types import base_types_pb2
 from kipy.proto.common.types.base_types_pb2 import LockedState
-from kipy.proto.board import board_types_pb2
+from kipy.proto.board import board_commands_pb2, board_types_pb2
 from kipy.common_types import (
     GraphicAttributes,
     TextAttributes,
@@ -48,6 +51,8 @@ from kipy.geometry import (
     arc_end_angle,
 )
 from kipy.util import unpack_any
+from kipy.util.board_layer import is_copper_layer
+from kipy.util.units import from_mm
 from kipy.wrapper import Item, Wrapper
 
 # Re-exported protobuf enum types
@@ -69,10 +74,16 @@ from kipy.proto.board.board_types_pb2 import (  # noqa
     ZoneType,
 )
 
+from kipy.proto.board.board_commands_pb2 import (  # noqa
+    InactiveLayerDisplayMode,
+    NetColorDisplayMode,
+    BoardFlipMode,
+    RatsnestDisplayMode
+)
 
 class BoardItem(Item):
     @property
-    def id(self):
+    def id(self) -> KIID:
         return self.proto.id
 
 
@@ -83,6 +94,9 @@ class Net(Wrapper):
         if proto is not None:
             self._proto.CopyFrom(proto)
 
+    def __repr__(self) -> str:
+        return f"Net(name={self.name}, code={self.code})"
+
     @property
     def name(self) -> str:
         return self._proto.name
@@ -106,6 +120,12 @@ class Track(BoardItem):
         if proto is not None:
             self._proto.CopyFrom(proto)
 
+    def __repr__(self) -> str:
+        return (
+            f"Track(start={self.start}, end={self.end}, layer={BoardLayer.Name(self.layer)}, "
+            f"net={self.net.name})"
+        )
+
     @property
     def net(self) -> Net:
         return Net(self._proto.net)
@@ -160,6 +180,12 @@ class ArcTrack(BoardItem):
         if proto is not None:
             self._proto.CopyFrom(proto)
 
+    def __repr__(self) -> str:
+        return (
+            f"ArcTrack(start={self.start}, mid={self.mid}, end={self.end}, "
+            f"layer={BoardLayer.Name(self.layer)}, net={self.net.name})"
+        )
+
     @property
     def net(self) -> Net:
         return Net(self._proto.net)
@@ -306,6 +332,18 @@ class BoardSegment(BoardShape, Segment):
         assert self._proto.shape.WhichOneof("geometry") == "segment"
         Segment.__init__(self, self._proto.shape)
 
+    def __repr__(self) -> str:
+        net_repr = (
+            f", net={self.net.name}"
+            if is_copper_layer(self.layer)
+            and self._proto.HasField("net")
+            else ""
+        )
+        return (
+            f"BoardSegment(start={self.start}, end={self.end}, layer={BoardLayer.Name(self.layer)}"
+            f"{net_repr})"
+        )
+
 class BoardArc(BoardShape, Arc):
     """Represents a graphic arc (not a track) on a board or footprint"""
 
@@ -318,6 +356,18 @@ class BoardArc(BoardShape, Arc):
         assert self._proto.shape.WhichOneof("geometry") == "arc"
         Arc.__init__(self, self._proto.shape)
 
+    def __repr__(self) -> str:
+        net_repr = (
+            f", net={self.net.name}"
+            if is_copper_layer(self.layer)
+            and self._proto.HasField("net")
+            else ""
+        )
+        return (
+            f"BoardArc(start={self.start}, mid={self.mid}, end={self.end}, "
+            f"layer={BoardLayer.Name(self.layer)}{net_repr})"
+        )
+
 class BoardCircle(BoardShape, Circle):
     """Represents a graphic circle on a board or footprint"""
 
@@ -330,6 +380,18 @@ class BoardCircle(BoardShape, Circle):
         assert self._proto.shape.WhichOneof("geometry") == "circle"
         Circle.__init__(self, self._proto.shape)
 
+    def __repr__(self) -> str:
+        net_repr = (
+            f", net={self.net.name}"
+            if is_copper_layer(self.layer)
+            and self._proto.HasField("net")
+            else ""
+        )
+        return (
+            f"BoardCircle(center={self.center}, radius_point={self.radius_point}, "
+            f"layer={BoardLayer.Name(self.layer)}{net_repr})"
+        )
+
 class BoardRectangle(BoardShape, Rectangle):
     """Represents a graphic rectangle on a board or footprint"""
 
@@ -342,6 +404,18 @@ class BoardRectangle(BoardShape, Rectangle):
         assert self._proto.shape.WhichOneof("geometry") == "rectangle"
         Rectangle.__init__(self, self._proto.shape)
 
+    def __repr__(self) -> str:
+        net_repr = (
+            f", net={self.net.name}"
+            if is_copper_layer(self.layer)
+            and self._proto.HasField("net")
+            else ""
+        )
+        return (
+            f"BoardRectangle(top_left={self.top_left}, bottom_right={self.bottom_right}, "
+            f"layer={BoardLayer.Name(self.layer)}{net_repr}"
+        )
+
 class BoardPolygon(BoardShape, Polygon):
     """Represents a graphic polygon on a board or footprint"""
 
@@ -354,6 +428,18 @@ class BoardPolygon(BoardShape, Polygon):
         assert self._proto.shape.WhichOneof("geometry") == "polygon"
         Polygon.__init__(self, self._proto.shape)
 
+    def __repr__(self) -> str:
+        net_repr = (
+            f", net={self.net.name}"
+            if is_copper_layer(self.layer)
+            and self._proto.HasField("net")
+            else ""
+        )
+        return (
+            f"BoardPolygon(points={self.polygons}, layer={BoardLayer.Name(self.layer)}"
+            f"{net_repr})"
+        )
+
 class BoardBezier(BoardShape, Bezier):
     """Represents a graphic bezier curve on a board or footprint"""
 
@@ -366,6 +452,18 @@ class BoardBezier(BoardShape, Bezier):
         assert self._proto.shape.WhichOneof("geometry") == "bezier"
         Bezier.__init__(self, self._proto.shape)
 
+    def __repr__(self) -> str:
+        net_repr = (
+            f", net={self.net.name}"
+            if is_copper_layer(self.layer)
+            and self._proto.HasField("net")
+            else ""
+        )
+        return (
+            f"BoardBezier(start={self.start}, control1={self.control1}, control2={self.control2}, "
+            f"end={self.end}, layer={BoardLayer.Name(self.layer)}{net_repr})"
+        )
+
 def to_concrete_board_shape(shape: BoardShape) -> Optional[BoardShape]:
     cls = {
         "segment": BoardSegment,
@@ -392,6 +490,9 @@ class BoardText(BoardItem):
         if proto is not None:
             self._proto.CopyFrom(proto)
 
+    def __repr__(self) -> str:
+        return f"BoardText(value={self.value}, position={self.position}, layer={BoardLayer.Name(self.layer)})"
+
     def as_text(self) -> Text:
         """Returns a base Text object using the same data as this BoardText"""
         return Text(self._proto.text)
@@ -453,6 +554,12 @@ class BoardTextBox(BoardItem):
         if proto is not None:
             self._proto.CopyFrom(proto)
 
+    def __repr__(self) -> str:
+        return (
+            f"BoardTextBox(value={self.value}, top_left={self.top_left}, "
+            f"bottom_right={self.bottom_right}, layer={BoardLayer.Name(self.layer)})"
+        )
+
     def as_textbox(self) -> TextBox:
         """Returns a base TextBox object using the same data as this BoardText"""
         return TextBox(self._proto.textbox)
@@ -522,6 +629,9 @@ class Field(BoardItem):
         if proto is not None:
             self._proto.CopyFrom(proto)
 
+    def __repr__(self) -> str:
+        return f"Field(name={self.name}, text={self.text}, layer={BoardLayer.Name(self.layer)})"
+
     @property
     def field_id(self) -> int:
         return self._proto.id.id
@@ -542,6 +652,57 @@ class Field(BoardItem):
     def text(self, text: BoardText):
         self._proto.text.CopyFrom(text.proto)
 
+
+class ThermalSpokeSettings(Wrapper):
+    def __init__(
+        self,
+        proto: Optional[board_types_pb2.ThermalSpokeSettings] = None,
+        proto_ref: Optional[board_types_pb2.ThermalSpokeSettings] = None,
+    ):
+        self._proto = (
+            proto_ref
+            if proto_ref is not None
+            else board_types_pb2.ThermalSpokeSettings()
+        )
+
+        if proto is not None:
+            self._proto.CopyFrom(proto)
+
+    @property
+    def width(self) -> Optional[int]:
+        if self._proto.HasField("width"):
+            return self._proto.width.value_nm
+        return None
+
+    @width.setter
+    def width(self, width: int):
+        if width is None:
+            self._proto.ClearField("width")
+        else:
+            self._proto.width.value_nm = width
+
+    @property
+    def angle(self) -> Angle:
+        return Angle(self._proto.angle)
+
+    @angle.setter
+    def angle(self, angle: Angle):
+        self._proto.angle.CopyFrom(angle.proto)
+
+    @property
+    def gap(self) -> Optional[int]:
+        if self._proto.HasField("gap"):
+            return self._proto.gap.value_nm
+        return None
+
+    @gap.setter
+    def gap(self, gap: Optional[int]):
+        if gap is None:
+            self._proto.ClearField("gap")
+        else:
+            self._proto.gap.value_nm = gap
+
+
 class ZoneConnectionSettings(Wrapper):
     def __init__(
         self,
@@ -566,8 +727,8 @@ class ZoneConnectionSettings(Wrapper):
         self._proto.zone_connection = zone_connection
 
     @property
-    def thermal_spokes(self) -> board_types_pb2.ThermalSpokeSettings:
-        return self._proto.thermal_spokes
+    def thermal_spokes(self) -> ThermalSpokeSettings:
+        return ThermalSpokeSettings(self._proto.thermal_spokes)
 
 
 class SolderMaskOverrides(Wrapper):
@@ -962,6 +1123,12 @@ class Via(BoardItem):
         if proto is not None:
             self._proto.CopyFrom(proto)
 
+    def __repr__(self) -> str:
+        return (
+            f"Via(position={self.position}, net={self.net.name}, type={ViaType.Name(self.type)}, "
+            f"locked={self.locked})"
+        )
+
     @property
     def position(self) -> Vector2:
         return Vector2(self._proto.position)
@@ -1114,6 +1281,9 @@ class Footprint(Wrapper):
         if proto is not None:
             self._proto.CopyFrom(proto)
 
+    def __repr__(self) -> str:
+        return f"Footprint(id={self.id}, items={len(self.items)})"
+
     @property
     def id(self) -> LibraryIdentifier:
         return LibraryIdentifier(proto_ref=self._proto.id)
@@ -1169,6 +1339,9 @@ class FootprintInstance(BoardItem):
         if proto is not None:
             self._proto.CopyFrom(proto)
 
+    def __repr__(self) -> str:
+        return f"FootprintInstance(id={self.id}, pos={self.position}, layer={BoardLayer.Name(self.layer)})"
+
     @property
     def id(self) -> KIID:
         return self._proto.id
@@ -1301,6 +1474,23 @@ class Zone(BoardItem):
 
         if proto is not None:
             self._proto.CopyFrom(proto)
+        else:
+            # Set reasonable defaults from KiCad ZONE_SETTINGS for convenience
+            self.type = ZoneType.ZT_COPPER
+            self.min_thickness = from_mm(0.25)
+            self.min_island_area = 10 * from_mm(1) * from_mm(1)
+            self.island_mode = IslandRemovalMode.IRM_ALWAYS
+            self.border_style = ZoneBorderStyle.ZBS_DIAGONAL_EDGE
+            self.border_hatch_pitch = from_mm(0.5)
+
+    def __repr__(self) -> str:
+        if self.type == ZoneType.ZT_COPPER:
+            assert self.net is not None
+            return f"Copper Zone(net={self.net.name}, layers={self.layers})"
+        elif self.type == ZoneType.ZT_RULE_AREA:
+            return f"Rule Area Zone(name={self.name}, layers={self.layers})"
+
+        return f"Zone(name={self.name}, type={self.type}, layers={self.layers})"
 
     @property
     def type(self) -> ZoneType.ValueType:
@@ -1350,10 +1540,6 @@ class Zone(BoardItem):
     def filled(self) -> bool:
         return self._proto.filled
 
-    @filled.setter
-    def filled(self, filled: bool):
-        self._proto.filled = filled
-
     @property
     def locked(self) -> bool:
         return self._proto.locked == LockedState.LS_LOCKED
@@ -1384,28 +1570,53 @@ class Zone(BoardItem):
 
     @property
     def clearance(self) -> Optional[int]:
+        """The override (local) clearance for this filled copper zone"""
         if self.is_rule_area():
             return None
         return self._proto.copper_settings.clearance.value_nm
 
+    @clearance.setter
+    def clearance(self, clearance: int):
+        if self.is_rule_area():
+            raise ValueError("clearance does not apply to rule areas")
+        self._proto.copper_settings.clearance.value_nm = clearance
+
     @property
     def min_thickness(self) -> Optional[int]:
         if self.is_rule_area():
             return None
         return self._proto.copper_settings.min_thickness.value_nm
 
+    @min_thickness.setter
+    def min_thickness(self, thickness: int):
+        if self.is_rule_area():
+            raise ValueError("min thickness does not apply to rule areas")
+        self._proto.copper_settings.min_thickness.value_nm = thickness
+
     @property
     def island_mode(self) -> Optional[IslandRemovalMode.ValueType]:
         if self.is_rule_area():
             return None
         return self._proto.copper_settings.island_mode
 
+    @island_mode.setter
+    def island_mode(self, mode: IslandRemovalMode.ValueType):
+        if self.is_rule_area():
+            raise ValueError("island removal mode does not apply to rule areas")
+        self._proto.copper_settings.island_mode = mode
+
     @property
     def min_island_area(self) -> Optional[int]:
         if self.is_rule_area():
             return None
         return self._proto.copper_settings.min_island_area
 
+    @min_island_area.setter
+    def min_island_area(self, area: int):
+        if self.is_rule_area():
+            raise ValueError("minimum island area does not apply to rule areas")
+        self._proto.copper_settings.min_island_area = area
+
     @property
     def fill_mode(self) -> Optional[ZoneFillMode.ValueType]:
         if self.is_rule_area():
@@ -1424,6 +1635,22 @@ class Zone(BoardItem):
             return None
         return self._proto.copper_settings.teardrop
 
+    @property
+    def border_style(self) -> ZoneBorderStyle.ValueType:
+        return self._proto.border.style
+
+    @border_style.setter
+    def border_style(self, style: ZoneBorderStyle.ValueType):
+        self._proto.border.style = style
+
+    @property
+    def border_hatch_pitch(self) -> int:
+        return self._proto.border.pitch.value_nm
+
+    @border_hatch_pitch.setter
+    def border_hatch_pitch(self, value: int):
+        self._proto.border.pitch.value_nm = value
+
     def bounding_box(self) -> Box2:
         return self.outline.bounding_box()
 
@@ -1587,6 +1814,12 @@ class AlignedDimension(Dimension):
 
         assert self._proto.WhichOneof("dimension_style") == "aligned"
 
+    def __repr__(self) -> str:
+        return (
+            f"AlignedDimension(start={self.start}, end={self.end}, "
+            f"height={self.height}, extension_height={self.extension_height})"
+        )
+
     @property
     def start(self) -> Vector2:
         return Vector2(self._proto.aligned.start)
@@ -1629,6 +1862,9 @@ class OrthogonalDimension(Dimension):
 
         assert self._proto.WhichOneof("dimension_style") == "orthogonal"
 
+    def __repr__(self) -> str:
+        return f"OrthogonalDimension(start={self.start}, end={self.end}, alignment={self.alignment})"
+
     @property
     def start(self) -> Vector2:
         return Vector2(self._proto.orthogonal.start)
@@ -1678,6 +1914,12 @@ class RadialDimension(Dimension):
 
         assert self._proto.WhichOneof("dimension_style") == "radial"
 
+    def __repr__(self) -> str:
+        return (
+            f"RadialDimension(center={self.center}, radius_point={self.radius_point}, "
+            f"leader_length={self.leader_length})"
+        )
+
     @property
     def center(self) -> Vector2:
         return Vector2(self._proto.radial.center)
@@ -1712,6 +1954,12 @@ class LeaderDimension(Dimension):
 
         assert self._proto.WhichOneof("dimension_style") == "leader"
 
+    def __repr__(self) -> str:
+        return (
+            f"LeaderDimension(start={self.start}, end={self.end}, "
+            f"border_style={self.border_style})"
+        )
+
     @property
     def start(self) -> Vector2:
         return Vector2(self._proto.leader.start)
@@ -1746,6 +1994,9 @@ class CenterDimension(Dimension):
 
         assert self._proto.WhichOneof("dimension_style") == "center"
 
+    def __repr__(self) -> str:
+        return f"CenterDimension(center={self.center}, end={self.end})"
+
     @property
     def center(self) -> Vector2:
         return Vector2(self._proto.center.center)
@@ -1776,6 +2027,58 @@ def to_concrete_dimension(dimension: Dimension) -> Optional[Dimension]:
     return cls(dimension._proto) if cls is not None else None
 
 
+class BoardEditorAppearanceSettings(Wrapper):
+    def __init__(
+        self,
+        proto: Optional[board_commands_pb2.BoardEditorAppearanceSettings] = None,
+        proto_ref: Optional[board_commands_pb2.BoardEditorAppearanceSettings] = None,
+    ):
+        self._proto = (
+            proto_ref
+            if proto_ref is not None
+            else board_commands_pb2.BoardEditorAppearanceSettings()
+        )
+
+        if proto is not None:
+            self._proto.CopyFrom(proto)
+
+    @property
+    def inactive_layer_display(self) -> board_commands_pb2.InactiveLayerDisplayMode.ValueType:
+        """How layers other than the active (selected) layer are displayed"""
+        return self._proto.inactive_layer_display
+
+    @inactive_layer_display.setter
+    def inactive_layer_display(self, mode: board_commands_pb2.InactiveLayerDisplayMode.ValueType):
+        self._proto.inactive_layer_display = mode
+
+    @property
+    def net_color_display(self) -> board_commands_pb2.NetColorDisplayMode.ValueType:
+        """Whether to apply net and netclass colors to copper items and ratsnest lines"""
+        return self._proto.net_color_display
+
+    @net_color_display.setter
+    def net_color_display(self, mode: board_commands_pb2.NetColorDisplayMode.ValueType):
+        self._proto.net_color_display = mode
+
+    @property
+    def board_flip(self) -> board_commands_pb2.BoardFlipMode.ValueType:
+        """Whether or not the board view is flipped (mirrored around the X axis)"""
+        return self._proto.board_flip
+
+    @board_flip.setter
+    def board_flip(self, mode: board_commands_pb2.BoardFlipMode.ValueType):
+        self._proto.board_flip = mode
+
+    @property
+    def ratsnest_display(self) -> board_commands_pb2.RatsnestDisplayMode.ValueType:
+        """Whether or not ratsnest lines are drawn to hidden layers"""
+        return self._proto.ratsnest_display
+
+    @ratsnest_display.setter
+    def ratsnest_display(self, mode: board_commands_pb2.RatsnestDisplayMode.ValueType):
+        self._proto.ratsnest_display = mode
+
+
 _proto_to_object: Dict[type[Message], type[Wrapper]] = {
     board_types_pb2.Arc: ArcTrack,
     board_types_pb2.BoardGraphicShape: BoardShape,
