@@ -1,19 +1,22 @@
-# This program source code file is part of KiCad, a free EDA CAD application.
+# Copyright The KiCad Developers
 #
-# Copyright (C) 2024 KiCad Developers
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the “Software”), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
 #
-# This program is free software: you can redistribute it and/or modify it
-# under the terms of the GNU General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or (at your
-# option) any later version.
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
 #
-# This program is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License along
-# with this program.  If not, see <http://www.gnu.org/licenses/>.
+# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
 
 from __future__ import annotations
 
@@ -85,8 +88,8 @@ class Vector2(Wrapper):
 
     def __mul__(self, scalar: float) -> Vector2:
         r = Vector2(self._proto)
-        r.x = int(r.x * scalar)
-        r.y = int(r.y * scalar)
+        r.x = int(float(r.x) * scalar)
+        r.y = int(float(r.y) * scalar)
         return r
 
     def length(self) -> float:
@@ -193,6 +196,9 @@ class Box2:
         if size_proto is not None:
             self._size_proto.CopyFrom(size_proto)
 
+    def __repr__(self):
+        return f"Box2(pos={self.pos}, size={self.size})"
+
     @classmethod
     def from_xywh(cls, x_nm: int, y_nm: int, w_nm: int, h_nm: int):
         pos = Vector2.from_xy(x_nm, y_nm)
@@ -376,6 +382,18 @@ class PolyLineNode(Wrapper):
         if proto is not None:
             self._proto.CopyFrom(proto)
 
+    @staticmethod
+    def from_point(point: Vector2):
+        n = PolyLineNode()
+        n.point = point
+        return n
+
+    @staticmethod
+    def from_xy(x: int, y: int):
+        n = PolyLineNode()
+        n.point = Vector2.from_xy(x, y)
+        return n
+
     def __repr__(self):
         if self.has_point:
             return f"PolyLineNode(point={self.point})"
@@ -475,6 +493,10 @@ class PolygonWithHoles(Wrapper):
     def outline(self) -> PolyLine:
         return PolyLine(proto_ref=self._proto.outline)
 
+    @outline.setter
+    def outline(self, outline: PolyLine):
+        self._proto.outline.CopyFrom(outline._proto)
+
     @property
     def holes(self) -> list[PolyLine]:
         return [PolyLine(proto_ref=hole) for hole in self._proto.holes]
@@ -489,10 +511,10 @@ class PolygonWithHoles(Wrapper):
         if not self.outline.nodes:
             return Box2()
 
-        min_x = int('inf')
-        min_y = int('inf')
-        max_x = int('-inf')
-        max_y = int('-inf')
+        min_x = math.inf
+        min_y = math.inf
+        max_x = -math.inf
+        max_y = -math.inf
 
         for node in self.outline:
             if node.has_point:
@@ -508,7 +530,8 @@ class PolygonWithHoles(Wrapper):
                 max_y = max(max_y, box.pos.y + box.size.y)
 
         return Box2.from_pos_size(
-            Vector2.from_xy(min_x, min_y), Vector2.from_xy(max_x - min_x, max_y - min_y)
+            Vector2.from_xy(int(min_x), int(min_y)),
+            Vector2.from_xy(int(max_x - min_x), int(max_y - min_y)),
         )
 
     def move(self, delta: Vector2):
@@ -551,14 +574,14 @@ def arc_center(start: Vector2, mid: Vector2, end: Vector2) -> Optional[Vector2]:
     mid1, dir1 = perpendicular_bisector(start, mid)
     mid2, dir2 = perpendicular_bisector(mid, end)
 
-    det = dir1.x * dir2.y - dir1.y * dir2.x
+    det = float(dir1.x * dir2.y - dir1.y * dir2.x)
 
     if det == 0:
         return None
 
     # Intersect the two perpendicular bisectors to find the center
-    t = (mid2.x - mid1.x) * dir2.y - (mid2.y - mid1.y) * dir2.x / det
-    center = mid1 + dir1 * t
+    t = ((mid2.x - mid1.x) * dir2.y - (mid2.y - mid1.y) * dir2.x) / det
+    center = mid1 + (dir1 * t)
 
     return center
 
